package jobshop;

import java.util.Date;
import org.chocosolver.solver.Model;
import org.chocosolver.solver.Solution;
import org.chocosolver.solver.Solver;
import org.chocosolver.solver.constraints.ReificationConstraint;
import org.chocosolver.solver.variables.IntVar;
import org.chocosolver.solver.variables.BoolVar;
import org.chocosolver.util.ESat;

public class Jobshop {
 public static void main(String[] args) {
  
  // les données du problème
  // -----------------------
  
  int n=3;   // 3 jobs
  int m=3;   // 3 machines
  int h = 100; // horizon 
  
  int[][] M = new int[n][m];
  int[][] P = new int[n][m];     
  
  M[0][0]=1;  M[0][1]=2;  M[0][2]=3;
  M[1][0]=2;  M[1][1]=3;  M[1][2]=1;
  M[2][0]=3;  M[2][1]=1;  M[2][2]=2;
  
  P[0][0]=10;  P[0][1]=20;  P[0][2]=30;
  P[1][0]=5;   P[1][1]=4;   P[1][2]=10;
  P[2][0]=12;  P[2][1]=7;   P[2][2]=4;
  

  int[]SommeDuree = new int[n];
  SommeDuree[0]=60;
  SommeDuree[1]=19;
  SommeDuree[2]=23;
  
  int[][] Tab = new int[n][m];     
  Tab[0][0]=0;  Tab[0][1]=1;  Tab[0][2]=2;
  Tab[1][0]=3;  Tab[1][1]=4;  Tab[1][2]=5;
  Tab[2][0]=6;  Tab[2][1]=7;  Tab[2][2]=8;

  
  Model mon_modele = new Model("JobShop");       
   
  // 2. variables
  
  IntVar Cmax = mon_modele.intVar("Cmax", 0, h);
  
  IntVar[] date_debut = new IntVar[n*m];
  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
     int position = Tab[i][j];
     date_debut[position] = mon_modele.intVar("St_"+i+"_"+j, 0, h);
     }
  }
  
  IntVar[] date_fin = new IntVar[n*m];
  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
     int position = Tab[i][j];
     date_fin[position] = mon_modele.intVar("Ft_"+i+"_"+j, 0, h);
     }
  } 


  IntVar[] durees = new IntVar[n*m];
  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
         int position = Tab[i][j];
         durees[position] = mon_modele.intVar("p_"+i+"_"+j, P[i][j]);
     }
  }

    
  BoolVar[][] b = new BoolVar [n*m][n*m];
  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
        for (int u=0; u<n; u++){
            for (int v=0; v<m; v++){
                 int position1 = Tab[i][j];
                 int position2 = Tab[u][v];
                b[position1][position2] = mon_modele.boolVar("b_"+i+"_"+j+"_"+u+"_"+v);
            }
        }
     }
  }  
  

  
  
  // contrainte 4. relation entre date de debut et de fin
  int[] coeffs = new int[3];
  coeffs[0] = 1;	//Ft
  coeffs[1] = -1;	//St
  coeffs[2] = -1;	//P

  for (int i=0; i<n; i++)
  {
    for (int j=0; j<m; j++)
     {
       int cour = Tab[i][j];
       IntVar[] colonne = new IntVar[3]; 
       colonne[0] = date_fin[cour];
       colonne[1] = date_debut[cour];
       colonne[2] = durees[cour];
       mon_modele.scalar(colonne, coeffs,"=",0).post();
     }
   }

  
  // contrainte 5 : gamme
  for (int i=0; i<n; i++){
     for (int j=1; j<m; j++){
        int prec =Tab[i][j]-1;
        int cour = Tab[i][j];
        mon_modele.arithm(date_fin[prec], "<=",date_debut[cour]).post();
     }
  }  
    
  // contrainte 6 : disjonction 
  for (int i=0; i<n-1; i++)
  {  
     for (int j=0; j<m; j++)
     {
        int ope_i = Tab[i][j];
        for (int u=i+1; u<n; u++)
        {
            for (int v=0; v<m; v++)
            {
                int ope_u = Tab[u][v];    
                if (M[i][j]==M[u][v])
                {
                    mon_modele.ifThenElse(
                        b[ope_i][ope_u]  ,
                        mon_modele.arithm(date_fin[ope_i],"<=",date_debut[ope_u]),
                        mon_modele.arithm(date_fin[ope_u],"<=",date_debut[ope_i]));
                    
                    /*
                     mon_modele.reification(
                        b[ope_i][ope_u]  ,
                        mon_modele.arithm(date_fin[ope_i],"<=",date_debut[ope_u]));
                     
                     mon_modele.reification(
                        b[ope_u][ope_i]  ,
                         mon_modele.arithm(date_fin[ope_u],"<=",date_debut[ope_i]));
                     
                     mon_modele.arithm( b[ope_i][ope_u]  , "+",b[ope_u][ope_i], "=",1).post();
                    */
                }
            }
        } 
     }
  } 
  
  // fonction objectif
  for (int i = 0; i < n; i++) 
  {
    int valeur = i*m+m-1;
    System.out.println("valeur : "+ valeur);
    mon_modele.arithm(date_fin[i*m+m-1], "<=",Cmax).post();
  }
  

  mon_modele.setObjective(Model.MINIMIZE, Cmax);    
  
  Solver mon_solveur= mon_modele.getSolver();
  Solution solution = new Solution(mon_modele);   
 
  while (mon_solveur.solve())
  {
     solution.record(); 
     System.out.println(Cmax.toString());
  }
  
  if(solution != null)
     {
       System.out.println(solution.toString());
       int cout =(int) mon_solveur.getBestSolutionValue(); 
       System.out.println("Cmax = " + cout);
     }
      else
        System.out.println("pas de solution");
  
if (mon_solveur.isFeasible() == ESat.TRUE){
	int cout =(int) mon_solveur.getBestSolutionValue();
	System.out.println("Cmax = " + cout);

	for (int i=0; i<n; i++){
		for(int j=0; j<m; j++){
			int position = Tab[i][j];
			int deb = solution.getIntVal(date_debut[position]);
			int fin = solution.getIntVal(date_fin[position]);
			System.out.print(deb + "/" + fin);
			System.out.print("    ");
		}
		System.out.println();
	}
}
else
	System.out.println("pas de sol.faisable");
  
  
  
  
   }
 
 
 
 
 

}